# -*- coding: utf-8 -*-

import datetime
from django.http import HttpResponseRedirect, Http404
from django.contrib import messages
from django.shortcuts import render
from django.utils.translation import ugettext as _
from django.template.context_processors import csrf
from django.contrib.auth.decorators import login_required
from koalixcrm.djangoUserExtension.models import UserExtension
from koalixcrm.crm.exceptions import ReportingPeriodNotFound
from koalixcrm.crm.exceptions import UserIsNoHumanResource
from koalixcrm.djangoUserExtension.exceptions import UserExtensionMissing, TooManyUserExtensionsAvailable
from koalixcrm.crm.views.range_selection_form import RangeSelectionForm
from koalixcrm.crm.views.work_entry_formset import BaseWorkEntryFormset
from koalixcrm.crm.reporting.human_resource import HumanResource


@login_required
def work_report(request):
    try:
        human_resource = HumanResource.objects.filter(user=UserExtension.get_user_extension(request.user))
        if len(human_resource) == 0:
            error_message = "User "+request.user.__str__()+" is not registered as human resource"
            raise UserIsNoHumanResource(error_message)
        else:
            human_resource = HumanResource.objects.get(user=UserExtension.get_user_extension(request.user))
        if request.POST.get('post'):
            if 'cancel' in request.POST:
                return HttpResponseRedirect('/admin/')
            elif 'save' in request.POST:
                range_selection_form = RangeSelectionForm(request.POST)
                if range_selection_form.is_valid():
                    formset = BaseWorkEntryFormset.load_formset(range_selection_form,
                                                                request)
                    if not formset.is_valid():
                        c = {'range_selection_form': range_selection_form,
                             'formset': formset}
                        c.update(csrf(request))
                        return render(request, 'crm/admin/time_reporting.html', c)
                    else:
                        for form in formset:
                            form.update_work(request)
                        messages.success(request, _('you have successfully updated your work'))
                formset = BaseWorkEntryFormset.create_updated_formset(range_selection_form,
                                                                      human_resource)
                range_selection_form.update_from_input()
                c = {'range_selection_form': range_selection_form,
                     'formset': formset}
                c.update(csrf(request))
                return render(request, 'crm/admin/time_reporting.html', c)
            return HttpResponseRedirect('/admin/')
        else:
            datetime_now = datetime.datetime.today()
            to_date = (datetime_now + datetime.timedelta(days=30)).date()
            from_date = datetime_now.date()
            range_selection_form = RangeSelectionForm.create_range_selection_form(from_date, to_date)
            formset = BaseWorkEntryFormset.create_new_formset(from_date,
                                                              to_date,
                                                              human_resource)
            c = {'formset': formset,
                 'range_selection_form': range_selection_form}
            c.update(csrf(request))
            return render(request, 'crm/admin/time_reporting.html', c)
    except (UserExtensionMissing,
            ReportingPeriodNotFound,
            TooManyUserExtensionsAvailable,
            UserIsNoHumanResource) as e:
        if isinstance(e, UserExtensionMissing):
            return HttpResponseRedirect(e.view)
        elif isinstance(e, ReportingPeriodNotFound):
            return HttpResponseRedirect(e.view)
        elif isinstance(e, TooManyUserExtensionsAvailable):
            return HttpResponseRedirect(e.view)
        elif isinstance(e, UserIsNoHumanResource):
            return HttpResponseRedirect(e.view)
        else:
            raise Http404



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _
from koalixcrm.crm.product.currency import Currency
from koalixcrm.crm.product.unit import Unit
from koalixcrm.crm.contact.customer_group import CustomerGroup
from koalixcrm.crm.product.unit_transform import UnitTransform
from koalixcrm.crm.product.customer_group_transform import CustomerGroupTransform
from koalixcrm.crm.product.currency_transform import CurrencyTransform


class Price(models.Model):
    unit = models.ForeignKey(Unit,
                             on_delete=models.CASCADE,
                             blank=False,
                             verbose_name=_("Unit"))
    currency = models.ForeignKey(Currency,
                                 on_delete=models.CASCADE,
                                 verbose_name='Currency',
                                 blank=False,
                                 null=False)
    customer_group = models.ForeignKey(CustomerGroup,
                                       on_delete=models.CASCADE,
                                       verbose_name=_("Customer Group"),
                                       blank=True,
                                       null=True)
    price = models.DecimalField(max_digits=17,
                                decimal_places=2,
                                verbose_name=_("Price Per Unit"))
    valid_from = models.DateField(verbose_name=_("Valid from"),
                                  blank=True,
                                  null=True)
    valid_until = models.DateField(verbose_name=_("Valid until"),
                                   blank=True,
                                   null=True)

    def __str__(self):
        return str(self.id) + " " +str(self.price) + " " + str(self.currency.short_name)

    def is_valid_from_criteria_fulfilled(self, date):
        if self.valid_from is None:
            return True
        elif (self.valid_from - date).days <= 0:
            return True
        else:
            return False

    def is_valid_until_criteria_fulfilled(self, date):
        if self.valid_until is None:
            return True
        elif (date - self.valid_until).days <= 0:
            return True
        else:
            return False

    def is_customer_group_criteria_fulfilled(self, customer_group):
        if self.customer_group is None:
            return True
        elif self.customer_group == customer_group:
            return True
        else:
            return False

    def is_currency_criteria_fulfilled(self, currency):
        if self.currency == currency:
            return True
        else:
            return False

    def is_unit_criteria_fulfilled(self, unit):
        if self.unit == unit:
            return True
        else:
            return False

    def is_date_in_range(self, date):
        if (self.valid_from is None) and (self.valid_until is None):
            return True
        elif self.valid_until is None:
            if self.valid_from <= date:
                return True
            else:
                return False
        elif self.valid_from is None:
            if date <= self.valid_until:
                return True
            else:
                return False
        elif (self.valid_from <= date) and (date <= self.valid_until):
            return True
        else:
            return False

    def get_currency_transform_factor(self, currency, product_type):
        """check currency conditions and factor"""
        currency_factor = 0
        if self.currency == currency:
            currency_factor = 1
        else:
            currency_transform = CurrencyTransform.objects.get(from_currency=self.currency,
                                                               to_currency=currency,
                                                               product_type=product_type)
            if currency_transform:
                currency_factor = currency_transform.get_transform_factor()
        return currency_factor

    def get_unit_transform_factor(self, unit, product_type):
        """check unit conditions and factor"""
        unit_factor = 0
        if self.unit == unit:
            unit_factor = 1
        else:
            unit_transform = UnitTransform.objects.get(from_unit=self.unit,
                                                       to_unit=unit,
                                                       product_type=product_type)
            if unit_transform:
                unit_factor = unit_transform.get_transform_factor()
        return unit_factor

    def get_customer_group_transform_factor(self, customer, product_type):
        """The function searches through all customer_groups in which the customer is member of
        from these customer_groups, the function returns the customer_group with the perfect match
        or it returns the factor with the lowest transform factor

        Args:
            koalixcrm.crm.contact.customer customer
            koalixcrm.crm.product.product product

        Returns:
            Decimal factor

        Raises:
            No exceptions planned"""
        customer_group_factor = 0
        if self.customer_group is None:
            customer_group_factor = 1
        elif customer is not None:
            customer_groups = customer.is_member_of.all()
            if customer_groups is not None:
                for customer_group in customer_groups:
                    if self.customer_group == customer_group:
                        customer_group_factor = 1
                        # Stop for loop when a perfect match is found
                        break
                    else:
                        customer_group_transform = CustomerGroupTransform.objects.get(
                            from_customer_group=self.customer_group,
                            to_customer_group=customer_group,
                            product_type=product_type)
                        if customer_group_transform:
                            transform_factor = customer_group_transform.get_transform_factor()
                            if customer_group_factor > transform_factor or customer_group_factor == 0:
                                customer_group_factor = transform_factor
        return customer_group_factor

    class Meta:
        app_label = "crm"
        verbose_name = _('Price')
        verbose_name_plural = _('Prices')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-16 20:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0019_auto_20180112_2020'),
    ]

    operations = [
        migrations.DeleteModel(
            name='PurchaseOrder',
        ),
        migrations.DeleteModel(
            name='EmailAddressForPurchaseOrder',
        ),
        migrations.DeleteModel(
            name='PhoneAddressForPurchaseOrder',
        ),
        migrations.DeleteModel(
            name='PostalAddressForPurchaseOrder',
        ),
        migrations.DeleteModel(
            name='PurchaseOrderPosition',
        ),
    ]



# -*- coding: utf-8 -*-


class ProgrammingError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class NoObjectsToBeSerialzed(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class AccountingPeriodNotFound(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class TemplateSetMissingInAccountingPeriod(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)




# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-12 20:56
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models

class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0007_auto_20180702_1628'),
        ('accounting', '0008_auto_20181012_2056'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('crm', '0047_work_worked_hours'),
        ('subscriptions', '0004_auto_20181013_2213')
    ]

    operations = [
        migrations.AddField(
            model_name='customergrouptransform',
            name='product_backup',
            field=models.IntegerField(blank=True, null=True, verbose_name='Product Backup Identifier'),
        ),
        migrations.AddField(
            model_name='position',
            name='product_backup',
            field=models.IntegerField(blank=True, null=True, verbose_name='Product Backup Identifier'),
        ),
        migrations.AddField(
            model_name='unittransform',
            name='product_backup',
            field=models.IntegerField(blank=True, null=True, verbose_name='Product Backup Identifier'),
        ),
        migrations.AddField(
            model_name='price',
            name='product_backup',
            field=models.IntegerField(blank=True, null=True, verbose_name='Product Backup Identifier'),
        ),
    ]



# -*- coding: utf-8 -*-


class TemplateSetMissing(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class TemplateMissingInTemplateSet(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class TemplateSetMissingInContract(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class TemplateFOPConfigFileMissing(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class TemplateXSLTFileMissing(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class NoSerializationPatternFound(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class OpenInterestAccountMissing(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class IncompleteInvoice(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class InvoiceAlreadyRegistered(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class UserIsNoHumanResource(Exception):
    def __init__(self, value):
        self.value = value
        self.view = "/koalixcrm/crm/reporting/user_is_not_human_resource"

    def __str__(self):
        return repr(self.value)


class ReportingPeriodDoneDeleteNotPossible(Exception):
    def __init__(self, value=None):
        self.value = value
        self.view = "/koalixcrm/crm/reporting/reporting_period_done_delete_not_possible"

    def __str__(self):
        return repr(self.value)


class ReportingPeriodNotFound(Exception):
    def __init__(self, value):
        self.value = value
        self.view = "/koalixcrm/crm/reporting/reporting_period_missing"

    def __str__(self):
        return repr(self.value)



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _


class UnitTransform(models.Model):
    from_unit = models.ForeignKey('Unit',
                                  on_delete=models.CASCADE,
                                  verbose_name=_("From Unit"),
                                  blank=False,
                                  null=False,
                                  related_name="db_reltransfromfromunit")
    to_unit = models.ForeignKey('Unit',
                                on_delete=models.CASCADE,
                                verbose_name=_("To Unit"),
                                blank=False,
                                null=False,
                                related_name="db_reltransfromtounit")
    product_type = models.ForeignKey('ProductType',
                                     on_delete=models.CASCADE,
                                     blank=False,
                                     null=False,
                                     verbose_name=_("Product Type"))
    factor = models.DecimalField(verbose_name=_("Factor between From and To Unit"),
                                 blank=False,
                                 null=False,
                                 max_digits=17,
                                 decimal_places=2,)

    def transform(self, unit):
        if self.from_unit == unit:
            return self.to_unit
        else:
            return None

    def get_transform_factor(self):
        return self.factor

    def __str__(self):
        return "From " + self.from_unit.short_name + " to " + self.to_unit.short_name

    class Meta:
        app_label = "crm"
        verbose_name = _('Unit Transform')
        verbose_name_plural = _('Unit Transforms')


class UnitTransformInlineAdminView(admin.TabularInline):
    model = UnitTransform
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('', {
            'fields': ('from_unit',
                       'to_unit',
                       'factor',)
        }),
    )
    allow_add = True



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-11 19:30
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('crm', '0029_auto_20180611_1903'),
    ]

    operations = [
        migrations.AddField(
            model_name='generictasklink',
            name='date_of_creation',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now, verbose_name='Created at'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='generictasklink',
            name='last_modified_by',
            field=models.ForeignKey(default=2, on_delete=django.db.models.deletion.CASCADE, related_name='db_task_link_last_modified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*

from django.utils.translation import ugettext as _

PURPOSESADDRESSINCONTRACT = (
    ('D', _('Delivery Address')),
    ('B', _('Billing Address')),
    ('C', _('Contact Address')),
)

PURPOSESADDRESSINCUSTOMER = (
    ('H', _('Private')),
    ('O', _('Business')),
    ('P', _('Mobile Private')),
    ('B', _('Mobile Business')),
)

PURPOSESTEXTPARAGRAPHINDOCUMENTS = (
    ('BS', _('Before subject')),
    ('AS', _('After subject')),
    ('BT', _('Before total')),
    ('AT', _('After total')),
    ('BW', _('Before wishes')),
    ('AW', _('After wishes')),
    ('C1', _('Custom 1')),
    ('C2', _('Custom 2')),
    ('C3', _('Custom 3')),
    ('C4', _('Custom 4')),
)

PURPOSECALLINCUSTOMER = (
    ('F', _('First commercial call')),
    ('S', _('Planned commercial call')),
    ('A', _('Assistance call')),
)

PURPOSEVISITINCUSTOMER = (
    ('F', _('First commercial visit')),
    ('S', _('Installation')),
)


# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-07-02 16:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0038_auto_20180702_1628'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='person',
            name='prename',
        ),
        migrations.AddField(
            model_name='person',
            name='pre_name',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Pre-name'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-04-13 19:37
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0004_auto_20171210_2126'),
        ('contenttypes', '0002_remove_content_type_name'),
        ('crm', '0024_auto_20180122_2121'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmployeeAssignmentToTask',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('planned_effort', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='Effort')),
                ('employee', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.UserExtension')),
            ],
            options={
                'verbose_name': 'Employee Assignment',
                'verbose_name_plural': 'Employee Assignments',
            },
        ),
        migrations.CreateModel(
            name='GenericTaskLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Task Link',
                'verbose_name_plural': 'Task Links',
            },
        ),
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('short_description', models.CharField(blank=True, max_length=100, null=True, verbose_name='Description')),
                ('planned_start_date', models.DateField(verbose_name='Planned Start Date')),
                ('planned_end_date', models.DateField(verbose_name='Planned End Date')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('last_status_change', models.DateField(blank=True, verbose_name='Last Status Change')),
                ('project', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Contract', verbose_name='Contract')),
            ],
            options={
                'verbose_name': 'Task',
                'verbose_name_plural': 'Tasks',
            },
        ),
        migrations.CreateModel(
            name='TaskLinkType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=300, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
            ],
            options={
                'verbose_name': 'Task Link Type',
                'verbose_name_plural': 'Task Link Type',
            },
        ),
        migrations.CreateModel(
            name='TaskStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=250, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('is_done', models.BooleanField(verbose_name='Status represents task done')),
            ],
            options={
                'verbose_name': 'Task Status',
                'verbose_name_plural': 'Task Status',
            },
        ),
        migrations.CreateModel(
            name='Work',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(verbose_name='Date')),
                ('start_time', models.DateTimeField(verbose_name='Start Time')),
                ('stop_time', models.DateTimeField(verbose_name='Stop Time')),
                ('short_description', models.CharField(max_length=300, verbose_name='Short Description')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('employee', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.UserExtension')),
                ('task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Task', verbose_name='Task')),
            ],
            options={
                'verbose_name': 'Work',
                'verbose_name_plural': 'Work',
            },
        ),
        migrations.AlterField(
            model_name='position',
            name='position_number',
            field=models.PositiveIntegerField(validators=[django.core.validators.MinValueValidator(1)], verbose_name='Position Number'),
        ),
        migrations.AddField(
            model_name='task',
            name='status',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.TaskStatus', verbose_name='Task Status'),
        ),
        migrations.AddField(
            model_name='generictasklink',
            name='task',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Task', verbose_name='Task'),
        ),
        migrations.AddField(
            model_name='generictasklink',
            name='task_link_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.TaskLinkType', verbose_name='Task Link Type'),
        ),
        migrations.AddField(
            model_name='employeeassignmenttotask',
            name='task',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Task', verbose_name='Task'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.http import HttpResponseRedirect
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.plugin import *
from koalixcrm.crm.contact.contact import Contact, ContactCall, ContactVisit,\
    PeopleInlineAdmin, PostalAddressForContact, ContactPostalAddress, \
    ContactPhoneAddress, ContactEmailAddress, CityFilter, StateFilter
from koalixcrm.crm.documents.contract import Contract


class Customer(Contact):
    default_customer_billing_cycle = models.ForeignKey('CustomerBillingCycle',
                                                       on_delete=models.CASCADE,
                                                       verbose_name=_('Default Billing Cycle'))
    is_member_of = models.ManyToManyField("CustomerGroup",
                                          verbose_name=_('Is member of'),
                                          blank=True)
    is_lead = models.BooleanField(default=True)

    def create_contract(self, request):
        contract = Contract()
        contract.create_from_reference(self, request.user)
        return contract

    def create_invoice(self, request):
        contract = self.create_contract(request)
        invoice = contract.create_invoice()
        return invoice

    def create_quote(self, request):
        contract = self.create_contract(request)
        quote = contract.create_quote()
        return quote

    def is_in_group(self, customer_group):
        for customer_group_membership in self.is_member_of.all():
            if customer_group_membership.id == customer_group.id:
                return 1
        return 0

    class Meta:
        app_label = "crm"
        verbose_name = _('Customer')
        verbose_name_plural = _('Customers')

    def __str__(self):
        return str(self.id) + ' ' + self.name


class IsLeadFilter(admin.SimpleListFilter):
    title = _('Is lead')
    parameter_name = 'is_lead'

    def lookups(self, request, model_admin):
        return (
            ('lead', _('Lead')),
            ('customer', _('Customer')),
        )

    def queryset(self, request, queryset):
        if self.value() == 'lead':
            return queryset.filter(is_lead=True)
        elif self.value() == 'customer':
            return queryset.filter(is_lead=False)
        else:
            return queryset


class OptionCustomer(admin.ModelAdmin):
    list_display = ('id',
                    'name',
                    'default_customer_billing_cycle',
                    'get_state',
                    'get_town',
                    'date_of_creation',
                    'get_is_lead',)
    list_filter = ('is_member_of', StateFilter, CityFilter, IsLeadFilter)
    fieldsets = (('', {'fields': ('name',
                                  'default_customer_billing_cycle',
                                  'is_member_of',)}),)
    allow_add = True
    ordering = ('id',)
    search_fields = ('id', 'name')
    inlines = [ContactPostalAddress,
               ContactPhoneAddress,
               ContactEmailAddress,
               PeopleInlineAdmin,
               ContactCall,
               ContactVisit]
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("customerInline"))

    @staticmethod
    def get_postal_address(obj):
        return PostalAddressForContact.objects.filter(person=obj.id).first()

    def get_state(self, obj):
        address = self.get_postal_address(obj)
        return address.state if address is not None else None

    get_state.short_description = _("State")

    def get_town(self, obj):
        address = self.get_postal_address(obj)
        return address.town if address is not None else None

    get_town.short_description = _("City")

    @staticmethod
    def get_is_lead(obj):
        return obj.is_lead

    get_is_lead.short_description = _("Is Lead")

    def create_contract(self, request, queryset):
        for obj in queryset:
            contract = obj.create_contract(request)
            response = HttpResponseRedirect('/admin/crm/contract/' + str(contract.id))
            return response

    create_contract.short_description = _("Create Contract")

    @staticmethod
    def create_quote(self, request, queryset):
        for obj in queryset:
            quote = obj.create_quote(request)
            response = HttpResponseRedirect('/admin/crm/quote/' + str(quote.id))
        return response

    create_quote.short_description = _("Create Quote")

    @staticmethod
    def create_invoice(self, request, queryset):
        for obj in queryset:
            invoice = obj.create_invoice(request)
            response = HttpResponseRedirect('/admin/crm/invoice/' + str(invoice.id))
        return response

    create_invoice.short_description = _("Create Invoice")

    def save_model(self, request, obj, form, change):
        if change:
            obj.last_modified_by = request.user
        else:
            obj.last_modified_by = request.user
            obj.staff = request.user
        obj.save()

    actions = ['create_contract', 'create_invoice', 'create_quote']
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("customerActions"))



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _
from django.contrib import admin
from rest_framework import serializers


class EstimationStatus(models.Model):
    title = models.CharField(verbose_name=_("Title"),
                             max_length=250,
                             blank=False,
                             null=False)
    description = models.TextField(verbose_name=_("Text"),
                                   blank=True,
                                   null=True)
    is_obsolete = models.BooleanField(verbose_name=_("Status represents estimation is obsolete"),)

    class Meta:
        app_label = "crm"
        verbose_name = _('Estimation Status')
        verbose_name_plural = _('Estimation Status')

    def __str__(self):
        return str(self.id) + " " + str(self.title)


class EstimationStatusAdminView(admin.ModelAdmin):
    list_display = ('id',
                    'title',
                    'description',
                    'is_obsolete')

    fieldsets = (
        (_('Agreement Status'), {
            'fields': ('title',
                       'description',
                       'is_obsolete')
        }),
    )
    save_as = True


class EstimationStatusJSONSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = EstimationStatus
        fields = ('id',
                  'title',
                  'description',)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-06 20:34
from __future__ import unicode_literals

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('crm', '0026_auto_20180507_1957'),
    ]

    operations = [
        migrations.CreateModel(
            name='Call',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.TextField(verbose_name='Description')),
                ('date_of_creation', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('date_due', models.DateTimeField(blank=True, default=datetime.datetime.now, verbose_name='Date due')),
                ('last_modification', models.DateTimeField(auto_now=True, verbose_name='Last modified')),
                ('status', models.CharField(choices=[('P', 'Planned'), ('D', 'Delayed'), ('R', 'ToRecall'), ('F', 'Failed'), ('S', 'Success')], default='P', max_length=1, verbose_name='Status')),
            ],
        ),
        migrations.CreateModel(
            name='ContactPersonAssociation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contact', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='person_association', to='crm.Contact')),
            ],
            options={
                'verbose_name': 'Contacts',
                'verbose_name_plural': 'Contacts',
            },
        ),
        migrations.CreateModel(
            name='Person',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('prefix', models.CharField(blank=True, choices=[('F', 'Company'), ('W', 'Mrs'), ('H', 'Mr'), ('G', 'Ms')], max_length=1, null=True, verbose_name='Prefix')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('prename', models.CharField(blank=True, max_length=100, null=True, verbose_name='Prename')),
                ('email', models.EmailField(max_length=200, verbose_name='Email Address')),
                ('phone', models.CharField(max_length=20, verbose_name='Phone Number')),
                ('role', models.CharField(blank=True, max_length=100, null=True, verbose_name='Role')),
                ('companies', models.ManyToManyField(blank=True, through='crm.ContactPersonAssociation', to='crm.Contact', verbose_name='Works at')),
            ],
            options={
                'verbose_name': 'Person',
                'verbose_name_plural': 'People',
            },
        ),
        migrations.AddField(
            model_name='customer',
            name='is_lead',
            field=models.BooleanField(default=True),
        ),
        migrations.CreateModel(
            name='CallForContact',
            fields=[
                ('call_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='crm.Call')),
                ('purpose', models.CharField(choices=[('F', 'First commercial call'), ('S', 'Planned commercial call'), ('A', 'Assistance call')], max_length=1, verbose_name='Purpose')),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Contact')),
                ('cperson', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Person', verbose_name='Person')),
            ],
            options={
                'verbose_name': 'Call',
                'verbose_name_plural': 'Calls',
            },
            bases=('crm.call',),
        ),
        migrations.CreateModel(
            name='VisitForContact',
            fields=[
                ('call_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='crm.Call')),
                ('purpose', models.CharField(choices=[('F', 'First commercial visit'), ('S', 'Installation')], max_length=1, verbose_name='Purpose')),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Contact')),
                ('cperson', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Person', verbose_name='Person')),
                ('ref_call', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.CallForContact', verbose_name='Reference Call')),
            ],
            options={
                'verbose_name': 'Visit',
                'verbose_name_plural': 'Visits',
            },
            bases=('crm.call',),
        ),
        migrations.AddField(
            model_name='contactpersonassociation',
            name='person',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contact_association', to='crm.Person'),
        ),
        migrations.AddField(
            model_name='call',
            name='last_modified_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_calllstmodified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AddField(
            model_name='call',
            name='staff',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_relcallstaff', to=settings.AUTH_USER_MODEL, verbose_name='Staff'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-08 20:48
from __future__ import unicode_literals
from django.db import migrations, models
import django.db.models.deletion

class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0013_auto_20180105_2159'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='invoice',
            name='derived_from_quote',
        ),
        migrations.AddField(
            model_name='salescontract',
            name='derived_from_sales_contract',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.SalesContract'),
        ),
    ]



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import ProjectStatus


class DoneProjectStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProjectStatus

    title = "Done"
    description = "This project is done"
    is_done = True


class StartedProjectStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProjectStatus

    title = "Started"
    description = "This project has started"
    is_done = False



# -*- coding: utf-8 -*-

import pytz
from django.shortcuts import redirect, render


def set_timezone(request):
    if request.method == 'POST':
        request.session['django_timezone'] = request.POST['timezone']
        return redirect('/')
    else:
        return render(request, 'crm/admin/set_timezone.html', {'timezones': pytz.common_timezones})



# -*- coding: utf-8 -*-

from koalixcrm.djangoUserExtension.user_extension.document_template import *
from koalixcrm.djangoUserExtension.user_extension.template_set import *
from koalixcrm.djangoUserExtension.user_extension.user_extension import *
from koalixcrm.djangoUserExtension.user_extension.text_paragraph import *






# -*- coding: utf-8 -*-

from koalixcrm.accounting.accounting.account import Account
from koalixcrm.accounting.accounting.booking import Booking
from koalixcrm.accounting.accounting.booking import InlineBookings
from koalixcrm.accounting.accounting.accounting_period import AccountingPeriod
from koalixcrm.accounting.accounting.product_category import ProductCategory









# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _
from django.contrib import admin
from rest_framework import serializers


class TaskStatus(models.Model):
    title = models.CharField(verbose_name=_("Title"), max_length=250, blank=False, null=False)
    description = models.TextField(verbose_name=_("Text"), blank=True, null=True)
    is_done = models.BooleanField(verbose_name=_("Status represents task done"),)

    class Meta:
        app_label = "crm"
        verbose_name = _('Task Status')
        verbose_name_plural = _('Task Status')

    def __str__(self):
        return str(self.id) + " " + str(self.title)


class OptionTaskStatus(admin.ModelAdmin):
    list_display = ('id',
                    'title',
                    'description',
                    'is_done')

    fieldsets = (
        (_('Task Status'), {
            'fields': ('title',
                       'description',
                       'is_done')
        }),
    )
    save_as = True


class TaskStatusJSONSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = TaskStatus
        fields = ('id',
                  'title',
                  'description',)



# -*- coding: utf-8 -*-
from rest_framework import serializers
from koalixcrm.crm.reporting.estimation import Estimation
from koalixcrm.crm.reporting.estimation_status import EstimationStatus
from koalixcrm.crm.reporting.resource import Resource
from koalixcrm.crm.reporting.task import Task
from koalixcrm.crm.rest.task_rest import OptionTaskJSONSerializer
from koalixcrm.crm.rest.estimation_status_rest import OptionEstimationStatusJSONSerializer
from koalixcrm.crm.rest.resource_rest import OptionResourceJSONSerializer


class EstimationJSONSerializer(serializers.HyperlinkedModelSerializer):
    task = OptionTaskJSONSerializer(source='task', allow_null=False)
    resource = OptionResourceJSONSerializer(source='resource', allow_null=False)
    status = OptionEstimationStatusJSONSerializer(source='status', allow_null=False)
    dateFrom = serializers.DateField(source='date_from')
    dateUntil = serializers.DateField(source='date_until')
    amount = serializers.DecimalField(source='amount')

    class Meta:
        model = Estimation
        fields = ('task',
                  'resource',
                  'amount',
                  'date_from',
                  'date_until',
                  'status',
                  'reporting_period')

    def create(self, validated_data):
        estimation = Estimation()
        estimation.amount = validated_data['amount']
        estimation.date_from = validated_data['dateFrom']
        estimation.date_until = validated_data['dateUntil']
        # Deserialize task
        task = validated_data.pop('task')
        if task:
            if task.get('id', None):
                estimation.task = Task.objects.get(id=task.get('id', None))
            else:
                estimation.task = None
        # Deserialize resource
        resource = validated_data.pop('resource')
        if resource:
            if resource.get('id', None):
                estimation.resource = Resource.objects.get(id=resource.get('id', None))
            else:
                estimation.resource = None
        # Deserialize status
        status = validated_data.pop('status')
        if status:
            if type.get('id', None):
                estimation.status = EstimationStatus.objects.get(id=status.get('id', None))
            else:
                estimation.status = None

        estimation.save()
        return estimation

    def update(self, estimation, validated_data):
        estimation.amount = validated_data['amount']
        estimation.date_from = validated_data['dateFrom']
        estimation.date_until = validated_data['dateUntil']
        task = validated_data.pop('task')
        if task:
            if task.get('id', estimation.task):
                estimation.task = Task.objects.get(id=task.get('id', None))
            else:
                estimation.task = estimation.task_id
        else:
            estimation.task = None
        # Deserialize resource
        resource = validated_data.pop('resource')
        if resource:
            if resource.get('id', estimation.resource):
                estimation.resource = Resource.objects.get(id=resource.get('id', None))
            else:
                estimation.resource = estimation.resource_id
        else:
            estimation.resource = None
        # Deserialize status
        status = validated_data.pop('status')
        if status:
            if status.get('id', estimation.status):
                estimation.status = EstimationStatus.objects.get(id=status.get('id', None))
            else:
                estimation.status = estimation.status_id
        else:
            estimation.status = None

        estimation.save()

        return estimation






# -*- coding: utf-8 -*-

from datetime import *
from django import forms
from django.db import models
from django.contrib import admin
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from django.utils.html import format_html
from django.contrib.admin import helpers
from django.shortcuts import render
from django.contrib import messages
from django.template.context_processors import csrf
from koalixcrm.crm.const.status import *
from koalixcrm.crm.exceptions import *
from koalixcrm import accounting
from koalixcrm.crm.documents.sales_document import SalesDocument, OptionSalesDocument
from koalixcrm.crm.documents.sales_document_position import SalesDocumentPosition
from koalixcrm.plugin import *
from koalixcrm.accounting.models import Account
from koalixcrm.global_support_functions import limit_string_length


class Invoice(SalesDocument):
    payable_until = models.DateField(verbose_name=_("To pay until"))
    payment_bank_reference = models.CharField(verbose_name=_("Payment Bank Reference"), max_length=100, blank=True,
                                              null=True)
    status = models.CharField(max_length=1, choices=INVOICESTATUS)

    def link_to_invoice(self):
        if self.id:
            return format_html("<a href='/admin/crm/invoice/%s' >%s</a>" % (str(self.id),
                                                                            limit_string_length(str(self.description),
                                                                                                30)))
        else:
            return "Not present"
    link_to_invoice.short_description = _("Invoice")

    def create_from_reference(self, calling_model):
        self.create_sales_document(calling_model)
        self.status = 'C'
        self.payable_until = date.today() + \
                             timedelta(days=self.customer.default_customer_billing_cycle.time_to_payment_date)
        self.date_of_creation = date.today().__str__()
        self.template_set = self.contract.get_template_set(self)
        self.save()
        self.attach_sales_document_positions(calling_model)
        self.attach_text_paragraphs()

    def register_invoice_in_accounting(self, request):
        dict_prices = dict()
        dict_tax = dict()
        current_valid_accounting_period = accounting.models.AccountingPeriod.get_current_valid_accounting_period()
        activa_account = accounting.models.Account.objects.filter(isopeninterestaccount=True)
        if not self.is_complete_with_price():
            raise IncompleteInvoice(_("Complete invoice and run price recalculation. Price may not be Zero"))
        if len(activa_account) == 0:
            raise OpenInterestAccountMissing(_("Please specify one open interest account in the accounting"))
        for position in list(SalesDocumentPosition.objects.filter(sales_document=self.id)):
            profit_account = position.product.accounting_product_categorie.profitAccount
            dict_prices[profit_account] = position.last_calculated_price
            dict_tax[profit_account] = position.last_calculated_tax

        for booking in accounting.models.Booking.objects.filter(accountingPeriod=current_valid_accounting_period):
            if booking.bookingReference == self:
                raise InvoiceAlreadyRegistered("The invoice is already registered")
        for profit_account, amount in iter(dict_prices.items()):
            booking = accounting.models.Booking()
            booking.toAccount = activa_account[0]
            booking.fromAccount = profit_account
            booking.bookingReference = self
            booking.accountingPeriod = current_valid_accounting_period
            booking.bookingDate = date.today().__str__()
            booking.staff = request.user
            booking.amount = amount
            booking.lastmodifiedby = request.user
            booking.save()

    def register_payment_in_accounting(self, request, amount, payment_account):
        current_valid_accounting_period = accounting.models.AccountingPeriod.get_current_valid_accounting_period()
        activa_account = accounting.models.Account.objects.filter(isopeninterestaccount=True)
        booking = accounting.models.Booking()
        booking.toAccount = payment_account
        booking.fromAccount = activa_account[0]
        booking.bookingDate = date.today().__str__()
        booking.bookingReference = self
        booking.accountingPeriod = current_valid_accounting_period
        booking.amount = amount
        booking.staff = request.user
        booking.lastmodifiedby = request.user
        booking.save()

    def __str__(self):
        return _("Invoice") + ": " + self.id.__str__() + " " + _("from Contract") + ": " + self.contract.id.__str__()

    class Meta:
        app_label = "crm"
        verbose_name = _('Invoice')
        verbose_name_plural = _('Invoices')


class OptionInvoice(OptionSalesDocument):
    list_display = OptionSalesDocument.list_display + ('payable_until', 'status',)
    list_filter = OptionSalesDocument.list_filter + ('status',)
    ordering = OptionSalesDocument.ordering
    search_fields = OptionSalesDocument.search_fields
    fieldsets = OptionSalesDocument.fieldsets + (
        (_('Invoice specific'), {
            'fields': ('payable_until', 'status', 'payment_bank_reference' )
        }),
    )

    class PaymentForm(forms.Form):
        payment_amount = forms.DecimalField()
        _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
        payment_account = forms.ModelChoiceField(Account.objects.filter(account_type="A"))

    def register_invoice_in_accounting(self, request, queryset):
        try:
            for obj in queryset:
                obj.register_invoice_in_accounting(request)
            self.message_user(request, _("Successfully registered Invoice in the Accounting"))
            return;
        except OpenInterestAccountMissing as e:
            self.message_user(request, "Did not register Invoice in Accounting: " + e.__str__(), level=messages.ERROR)
            return;
        except IncompleteInvoice as e:
            self.message_user(request, "Did not register Invoice in Accounting: " + e.__str__(), level=messages.ERROR)
            return;

    register_invoice_in_accounting.short_description = _("Register Invoice in Accounting")

    # def unregisterInvoiceInAccounting(self, request, queryset):
    # for obj in queryset:
    # obj.createPDF(deliveryorder=True)
    # self.message_user(request, _("Successfully unregistered Invoice in the Accounting"))
    # unregisterInvoiceInAccounting.short_description = _("Unregister Invoice in Accounting")

    def register_payment_in_accounting(self, request, queryset):
        form = None
        if request.POST.get('post'):
            if 'cancel' in request.POST:
                self.message_user(request, _("Canceled registration of payment in the accounting"), level=messages.ERROR)
                return
            elif 'register' in request.POST:
                form = self.PaymentForm(request.POST)
                if form.is_valid():
                    payment_amount = form.cleaned_data['payment_amount']
                    payment_account = form.cleaned_data['payment_account']
                    for obj in queryset:
                        obj.register_payment_in_accounting(request, payment_amount, payment_account)
                    self.message_user(request, _("Successfully registered Payment in the Accounting"))
                    return HttpResponseRedirect(request.get_full_path())
        else:
            form = self.PaymentForm
            c = {'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
                 'queryset': queryset,
                 'form': form}
            c.update(csrf(request))
            return render(request, 'crm/admin/register_payment.html', c)

    register_payment_in_accounting.short_description = _("Register Payment in Accounting")

    save_as = OptionSalesDocument.save_as
    inlines = OptionSalesDocument.inlines

    actions = ['create_purchase_confirmation',
               'create_quote',
               'create_invoice',
               'create_delivery_note',
               'create_purchase_order',
               'create_payment_reminder',
               'create_pdf',
               'register_invoice_in_accounting',
               'register_payment_in_accounting',]

    pluginProcessor = PluginProcessor()
    actions.extend(pluginProcessor.getPluginAdditions("invoiceActions"))
    inlines.extend(pluginProcessor.getPluginAdditions("invoiceInlines"))


class InlineInvoice(admin.TabularInline):
    model = Invoice
    classes = ['collapse']
    show_change_link = True
    can_delete = True
    extra = 1
    readonly_fields = ('link_to_invoice',
                       'contract',
                       'customer',
                       'payable_until',
                       'status',
                       'last_pricing_date',
                       'last_calculated_price',
                       'last_calculated_tax')
    fieldsets = (
        (_('Invoice'), {
            'fields': ('link_to_invoice',
                       'contract',
                       'customer',
                       'payable_until',
                       'status',
                       'last_pricing_date',
                       'last_calculated_price',
                       'last_calculated_tax')
        }),
    )

    allow_add = False



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _

from koalixcrm.crm.const.purpose import *


class TextParagraphInDocumentTemplate(models.Model):
    document_template = models.ForeignKey("djangoUserExtension.DocumentTemplate", on_delete=models.CASCADE)
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=2, choices=PURPOSESTEXTPARAGRAPHINDOCUMENTS)
    text_paragraph = models.TextField(verbose_name=_("Text"), blank=False, null=False)

    class Meta:
        app_label = "crm"
        verbose_name = _('TextParagraphInDocumentTemplate')
        verbose_name_plural = _('TextParagraphInDocumentTemplates')

    def __str__(self):
        return str(self.id)


class InlineTextParagraph(admin.TabularInline):
    model = TextParagraphInDocumentTemplate
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('purpose', 'text_paragraph',)
        }),
    )
    allow_add = True


# -*- coding: utf-8 -*-
from rest_framework import serializers
from koalixcrm.crm.reporting.project import Project
from koalixcrm.crm.product.currency import Currency
from koalixcrm.crm.reporting.project_status import ProjectStatus
from koalixcrm.crm.rest.project_status_rest import OptionProjectStatusJSONSerializer
from koalixcrm.crm.rest.currency_rest import CurrencyJSONSerializer
from koalixcrm.djangoUserExtension.rest.user_rest import UserSerializer
from koalixcrm.djangoUserExtension.rest.template_set_rest import OptionTemplateSetJSONSerializer
from koalixcrm.djangoUserExtension.user_extension.template_set import TemplateSet
import koalixcrm


class OptionProjectJSONSerializer(serializers.ModelSerializer):
    projectStatus = OptionProjectStatusJSONSerializer(source='project_status', read_only=True)
    projectManager = UserSerializer(source='project_manager', read_only=True)
    projectName = serializers.CharField(source='project_name', read_only=True)
    defaultCurrency = CurrencyJSONSerializer(source='default_currency', read_only=True)
    defaultTemplateSet = OptionTemplateSetJSONSerializer(source='default_template_set', read_only=True)
    isReportingAllowed = serializers.SerializerMethodField()

    def get_isReportingAllowed(self, obj):
        if obj.is_reporting_allowed():
            return "True"
        else:
            return "False"

    class Meta:
        model = Project
        fields = ('projectStatus',
                  'projectManager',
                  'projectName',
                  'description',
                  'defaultCurrency',
                  'defaultTemplateSet',
                  'isReportingAllowed')


class ProjectJSONSerializer(serializers.ModelSerializer):
    from koalixcrm.crm.rest.task_rest import TaskJSONSerializer
    projectStatus = OptionProjectStatusJSONSerializer(source='project_status')
    projectManager = UserSerializer(source='project_manager', read_only=True)
    projectName = serializers.CharField(source='project_name')
    defaultCurrency = CurrencyJSONSerializer(source='default_currency')
    defaultTemplateSet = OptionTemplateSetJSONSerializer(source='default_template_set')
    isReportingAllowed = serializers.SerializerMethodField()
    tasks = TaskJSONSerializer(many=True, read_only=True)

    class Meta:
        model = Project
        fields = ('projectStatus',
                  'projectManager',
                  'projectName',
                  'description',
                  'defaultCurrency',
                  'defaultTemplateSet',
                  'isReportingAllowed',
                  'tasks')

    def get_isReportingAllowed(self, obj):
        if obj.is_reporting_allowed():
            return "True"
        else:
            return "False"

    def create(self, validated_data):
        project = Project()
        # Deserialize default currency
        default_currency = validated_data.pop('defaultCurrency')
        if default_currency:
            if default_currency.get('id', None):
                project.default_currency = Currency.objects.get(id=default_currency.get('id', None))
            else:
                project.default_currency = None
        # Deserialize status
        project_status = validated_data.pop('ProjectStatus')
        if project_status:
            if project_status.get('id', None):
                project.project_status = ProjectStatus.objects.get(id=project_status.get('id', None))
            else:
                project.project_status = None
        # Deserialize default template set
        default_template_set = validated_data.pop('defaultTemplateSet')
        if default_template_set:
            if default_template_set.get('id', None):
                project.default_template_set = TemplateSet.objects.get(id=default_template_set.get('id', None))
            else:
                project.default_template_set = None
        project.title = validated_data['title']
        project.description = validated_data['description']
        project.save()
        return project

    def update(self, project, validated_data):
        # Deserialize default currency
        default_currency = validated_data.pop('defaultCurrency')
        if default_currency:
            if default_currency.get('id', project.project):
                project.default_currency = Project.objects.get(id=default_currency.get('id', None))
            else:
                project.default_currency = project.default_currency_id
        else:
            project.default_currency = None
        # Deserialize status
        project_status = validated_data.pop('status')
        if project_status:
            if project_status.get('id', project.status):
                project.project_status = ProjectStatus.objects.get(id=project_status.get('id', None))
            else:
                project.project_status = project.project_status_id
        else:
            project.project_status = None
        # Deserialize default template set
        default_template_set = validated_data.pop('status')
        if default_template_set:
            if default_template_set.get('id', project.default_template_set):
                project.default_template_set = TemplateSet.objects.get(id=default_template_set.get('id', None))
            else:
                project.default_template_set = project.default_template_set_id
        else:
            project.default_template_set = None
        project.title = validated_data['title']
        project.description = validated_data['description']
        project.save()
        return project






# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-22 19:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0054_auto_20181020_1845'),
    ]

    operations = [
        migrations.AlterField(
            model_name='currencytransform',
            name='factor',
            field=models.DecimalField(decimal_places=2, default=1, max_digits=17, verbose_name='Factor between From and To Currency'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='customergrouptransform',
            name='factor',
            field=models.DecimalField(decimal_places=2, default=1, max_digits=17, verbose_name='Factor between From and To Customer Group'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='unittransform',
            name='factor',
            field=models.DecimalField(decimal_places=2, default=1, max_digits=17, verbose_name='Factor between From and To Unit'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-07-24 18:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0041_auto_20180724_1657'),
    ]

    operations = [
        migrations.AlterField(
            model_name='unit',
            name='fraction_factor_to_next_higher_unit',
            field=models.DecimalField(blank=True, decimal_places=10, max_digits=20, null=True, verbose_name='Factor Between This And Next Higher Unit'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-07-01 21:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0005_auto_20180612_1924'),
    ]

    operations = [
        migrations.CreateModel(
            name='WorkReportTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Work report template',
                'verbose_name_plural': 'Work report templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.AlterModelOptions(
            name='templateset',
            options={'verbose_name': 'Template-set', 'verbose_name_plural': 'Template-sets'},
        ),
        migrations.AlterModelOptions(
            name='userextension',
            options={'verbose_name': 'User Extension', 'verbose_name_plural': 'User Extension'},
        ),
        migrations.AlterModelOptions(
            name='userextensionemailaddress',
            options={'verbose_name': 'Email Address for User Extension', 'verbose_name_plural': 'Email Address for User Extension'},
        ),
        migrations.AlterModelOptions(
            name='userextensionphoneaddress',
            options={'verbose_name': 'Phone number for User Extension', 'verbose_name_plural': 'Phone number for User Extension'},
        ),
        migrations.AlterModelOptions(
            name='userextensionpostaladdress',
            options={'verbose_name': 'Postal Address for User Extension', 'verbose_name_plural': 'Postal Address for User Extension'},
        ),
        migrations.AddField(
            model_name='templateset',
            name='work_report_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.WorkReportTemplate'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-13 22:13
from __future__ import unicode_literals

from django.db import migrations, models
from filebrowser.fields import FileBrowseField


def fill_subscription_type_backup_table(apps, schema_editor):
    Subscription = apps.get_model("subscriptions", "Subscription")
    SubscriptionType = apps.get_model("subscriptions", "SubscriptionType")
    SubscriptionTypeBackup = apps.get_model("subscriptions", "SubscriptionTypeBackup")
    db_alias = schema_editor.connection.alias
    all_subscription_types = SubscriptionType.objects.using(db_alias).all()
    for subscription_type in all_subscription_types:
        subscription_type_backup = SubscriptionTypeBackup.objects.using(db_alias).create(
            cancellation_period=subscription_type.cancelationPeriod,
            automatic_contract_extension=subscription_type.automaticContractExtension,
            automatic_contract_extension_reminder=subscription_type.automaticContractExtensionReminder,
            minimum_duration=subscription_type.minimumDuration,
            payment_interval=subscription_type.paymentIntervall,
            contract_document=subscription_type.contractDocument,
            description=subscription_type.description,
            title=subscription_type.title,
            product_number=subscription_type.product_number,
            default_unit=subscription_type.default_unit,
            date_of_creation=subscription_type.date_of_creation,
            last_modification=subscription_type.last_modification,
            last_modified_by=subscription_type.last_modified_by,
            tax=subscription_type.tax,
            old_id=subscription_type.id)
        subscription_type_backup.save()
        subscriptions = Subscription.objects.using(db_alias).all()
        if len(subscriptions) > 0:
            for subscription in subscriptions:
                if subscription.subscriptiontype is not None:
                    subscription.subscription_type_backup = subscription.subscriptiontype.id
                    subscription.save()


def reverse_func(apps, schema_editor):
    return 1


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0047_work_worked_hours'),
        ('subscriptions', '0003_auto_20171110_1732'),
    ]

    operations = [
        migrations.RenameField(
            model_name='subscriptionevent',
            old_name='eventdate',
            new_name='event_date',
        ),
        migrations.AddField(
            model_name='Subscription',
            name='subscription_type_backup',
            field=models.IntegerField(blank=True, null=True, verbose_name='Subscription Type Backup'),
        ),
        migrations.CreateModel(
            name='SubscriptionTypeBackup',
            fields=[
                ('cancellation_period', models.IntegerField(verbose_name="Cancellation Period (months)",
                                                            blank=True,
                                                            null=True)),
                ('automatic_contract_extension', models.IntegerField(
                    verbose_name="Automatic Contract Extension (months)",
                    blank=True,
                    null=True)),
                ('automatic_contract_extension_reminder', models.IntegerField(
                    verbose_name="Automatic Contract Extension Reminder (days)",
                    blank=True,
                    null=True)),
                ('minimum_duration', models.IntegerField(verbose_name="Minimum Contract Duration",
                                                         blank=True,
                                                         null=True)),
                ('payment_interval', models.IntegerField(verbose_name="Payment Interval (days)",
                                                         blank=True,
                                                         null=True)),
                ('contract_document', FileBrowseField(verbose_name="Contract Documents",
                                                      blank=True,
                                                      null=True,
                                                      max_length=200)),
                ('description', models.TextField(verbose_name="Description",
                                                 null=True,
                                                 blank=True)),
                ('title', models.CharField(verbose_name="Title",
                                           max_length=200)),
                ('product_number', models.IntegerField(verbose_name="Product Number")),
                ('default_unit', models.ForeignKey("crm.Unit", on_delete=models.deletion.CASCADE,
                                                   verbose_name="Unit")),
                ('date_of_creation', models.DateTimeField(verbose_name="Created at",
                                                          auto_now_add=True)),
                ('last_modification', models.DateTimeField(verbose_name="Last modified",
                                                           auto_now=True)),
                ('last_modified_by', models.ForeignKey('auth.User',
                                                       on_delete=models.deletion.CASCADE,
                                                       limit_choices_to={'is_staff': True},
                                                       verbose_name="Last modified by",
                                                       null=True,
                                                       blank="True")),
                ('tax', models.ForeignKey("crm.Tax",
                                          on_delete=models.deletion.CASCADE,
                                          blank=False)),
                ('old_id', models.IntegerField(blank=True,
                                               null=True,
                                               verbose_name='Subscription Type Backup')),
            ],
            options={
                'verbose_name': 'SubscriptionTypeBackup',
                'verbose_name_plural': 'SubscriptionTypeBackups',
            },
        ),
        migrations.RunPython(fill_subscription_type_backup_table, reverse_func),
        migrations.RemoveField(
            model_name='subscription',
            name='subscriptiontype',
        ),
        migrations.DeleteModel('SubscriptionType'),
    ]



import datetime
import pytest
from django.test import TestCase
from koalixcrm.crm.factories.factory_user import AdminUserFactory
from koalixcrm.crm.factories.factory_customer_billing_cycle import StandardCustomerBillingCycleFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.crm.factories.factory_unit import StandardUnitFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_reporting_period import StandardReportingPeriodFactory
from koalixcrm.crm.factories.factory_human_resource import StandardHumanResourceFactory
from koalixcrm.crm.factories.factory_work import StandardWorkFactory
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.crm.factories.factory_resource_price import StandardResourcePriceFactory
from koalixcrm.crm.factories.factory_estimation import StandardHumanResourceEstimationToTaskFactory
from koalixcrm.crm.factories.factory_agreement import StandardAgreementToTaskFactory
from koalixcrm.global_support_functions import make_date_utc


class TaskEffectiveCostsWithAgreement(TestCase):
    def setUp(self):
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        start_date = (datetime_now - datetime.timedelta(days=30)).date()
        end_date_first_task = (datetime_now + datetime.timedelta(days=30)).date()
        end_date_second_task = (datetime_now + datetime.timedelta(days=60)).date()

        self.test_billing_cycle = StandardCustomerBillingCycleFactory.create()
        self.test_user = AdminUserFactory.create()
        self.test_unit = StandardUnitFactory.create()
        self.test_customer_group = StandardCustomerGroupFactory.create()
        self.test_customer = StandardCustomerFactory.create(is_member_of=(self.test_customer_group,))
        self.test_currency = StandardCurrencyFactory.create(
            rounding="0.05"
        )
        self.human_resource = StandardHumanResourceFactory.create()
        self.human_resource_two = StandardHumanResourceFactory.create()
        self.resource_price = StandardResourcePriceFactory.create(
            resource=self.human_resource,
            unit=self.test_unit,
            currency=self.test_currency,
            customer_group=self.test_customer_group,
            price="120",
        )
        self.resource_price_agreement = StandardResourcePriceFactory.create(
            resource=self.human_resource_two,
            unit=self.test_unit,
            currency=self.test_currency,
            customer_group=self.test_customer_group,
            price="90",
        )
        self.test_reporting_period = StandardReportingPeriodFactory.create()
        self.test_1st_task = StandardTaskFactory.create(
            title="1st Test Task",
            project=self.test_reporting_period.project
        )
        self.agreement_1st_task = StandardAgreementToTaskFactory(
            amount="3.50",
            task=self.test_1st_task,
            resource=self.human_resource,
            unit=self.test_unit,
            costs=self.resource_price_agreement
        )
        self.estimation_1st_task = StandardHumanResourceEstimationToTaskFactory(
            resource=self.human_resource,
            task=self.test_1st_task,
            date_from=start_date,
            date_until=end_date_first_task,
            amount=20
        )
        self.test_2nd_task = StandardTaskFactory.create(
            title="2nd Test Task",
            project=self.test_reporting_period.project
        )
        self.estimation_2nd_task = StandardHumanResourceEstimationToTaskFactory(
            resource=self.human_resource,
            task=self.test_2nd_task,
            date_from=start_date,
            date_until=end_date_second_task,
            amount=30
        )

    @pytest.mark.back_end_tests
    def test_task_effective_costs_with_agreement(self):
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        datetime_later_1 = make_date_utc(datetime.datetime(2024, 1, 1, 2, 00))
        datetime_later_2 = make_date_utc(datetime.datetime(2024, 1, 1, 3, 30))
        datetime_later_3 = make_date_utc(datetime.datetime(2024, 1, 1, 5, 45))
        datetime_later_4 = make_date_utc(datetime.datetime(2024, 1, 1, 6, 15))
        date_now = datetime_now.date()
        self.assertEqual(
            (self.test_1st_task.planned_duration()).__str__(), "60")
        self.assertEqual(
            (self.test_1st_task.planned_costs()).__str__(), "2400.0000")
        self.assertEqual(
            (self.test_2nd_task.planned_duration()).__str__(), "90")
        self.assertEqual(
            (self.test_2nd_task.planned_costs()).__str__(), "3600.0000")
        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=date_now,
            start_time=datetime_now,
            stop_time=datetime_later_1,
            task=self.test_1st_task,
            reporting_period=self.test_reporting_period
        )
        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=date_now,
            start_time=datetime_later_1,
            stop_time=datetime_later_2,
            task=self.test_1st_task,
            reporting_period=self.test_reporting_period
        )
        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=date_now,
            start_time=datetime_now,
            stop_time=datetime_later_3,
            task=self.test_2nd_task,
            reporting_period=self.test_reporting_period
        )
        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=date_now,
            start_time=datetime_now,
            stop_time=datetime_later_4,
            task=self.test_2nd_task,
            reporting_period=self.test_reporting_period
        )
        self.assertEqual(
            (self.test_1st_task.effective_effort(reporting_period=None)).__str__(), "3.5")
        self.assertEqual(
            (self.test_1st_task.effective_costs(reporting_period=None, confirmed=False)).__str__(), "315.00")
        self.assertEqual(
            (self.test_2nd_task.effective_effort(reporting_period=None)).__str__(), "12")
        self.assertEqual(
            (self.test_2nd_task.effective_costs(reporting_period=None, confirmed=False)).__str__(), "1440.00")



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-07-24 16:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0040_auto_20180724_1657'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tasks', to='crm.Project', verbose_name='Project'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _


class CustomerGroup(models.Model):
    name = models.CharField(max_length=300)

    def __str__(self):
        return str(self.id) + ' ' + self.name

    class Meta:
        app_label = "crm"
        verbose_name = _('Customer Group')
        verbose_name_plural = _('Customer Groups')


class OptionCustomerGroup(admin.ModelAdmin):
    list_display = ('id', 'name')
    fieldsets = (('', {'fields': ('name',)}),)
    allow_add = True


# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-12 20:56
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


def initiate_human_resource(apps, schema_editor):
    Work = apps.get_model("crm", "Work")
    HumanResource = apps.get_model("crm", "HumanResource")
    db_alias = schema_editor.connection.alias
    all_works = Work.objects.using(db_alias).all()
    for work in all_works:
        all_human_resources = HumanResource.objects.using(db_alias).all()
        human_resource_found = False
        for human_resource in all_human_resources:
            if work.employee == human_resource.user:
                human_resource_found = True
                work.human_resource = human_resource
                break
        if not human_resource_found:
            new_human_resource = HumanResource.objects.using(db_alias).create(user=work.employee)
            new_human_resource.save()
            work.human_resource = new_human_resource
        work.save()


def reverse_func(apps, schema_editor):
    return 1


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0052_auto_20181014_2304'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customergrouptransform',
            name='product_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.ProductType', verbose_name='Product Type'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='unittransform',
            name='product_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.ProductType', verbose_name='Product Type'),
            preserve_default=False,
        ),
        migrations.RemoveField(
            model_name='customergrouptransform',
            name='product_backup',
        ),
        migrations.RemoveField(
            model_name='position',
            name='product_backup',
        ),
        migrations.RemoveField(
            model_name='price',
            name='product_backup',
        ),
        migrations.RemoveField(
            model_name='unittransform',
            name='product_backup',
        ),

        migrations.AddField(
            model_name='producttype',
            name='accounting_product_category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='accounting.ProductCategory', verbose_name='Accounting Product Category'),
            preserve_default=False,
        ),
        migrations.RemoveField(model_name='producttype',
                               name='accounting_product_categorie'),
        migrations.AlterModelOptions(
            name='producttype',
            options={'verbose_name': 'Product Type', 'verbose_name_plural': 'Product Types'},
        ),
        migrations.RemoveField(
            model_name='producttype',
            name='product_number',
        ),
        migrations.AddField(
            model_name='producttype',
            name='product_type_identifier',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Product Number'),
        ),
        migrations.AlterField(
            model_name='position',
            name='product_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.ProductType', verbose_name='Product'),
        ),
        migrations.AlterField(
            model_name='productprice',
            name='product_type',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='crm.ProductType', verbose_name='Product Type'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='producttype',
            name='last_modified_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.CharField(verbose_name="Product Number",
                                                max_length=200,
                                                null=True,
                                                blank=True)),
                ('product_type', models.ForeignKey("ProductType", on_delete=django.db.models.deletion.CASCADE, verbose_name='Product Type')),
            ],
            options={
                'verbose_name': 'Product',
                'verbose_name_plural': 'Products',
            },
        ),
        migrations.CreateModel(
            name='Agreement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_from', models.DateField(verbose_name='Agreement From')),
                ('date_until', models.DateField(verbose_name='Agreement To')),
                ('amount', models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Amount')),
            ],
            options={
                'verbose_name': 'Agreement',
                'verbose_name_plural': 'Agreements',
            },
        ),
        migrations.CreateModel(
            name='AgreementStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=250, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('is_agreed', models.BooleanField(verbose_name='Status represents agreement exists')),
            ],
            options={
                'verbose_name': 'Agreement Status',
                'verbose_name_plural': 'Agreement Status',
            },
        ),
        migrations.CreateModel(
            name='AgreementType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=300, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
            ],
            options={
                'verbose_name': 'Agreement Type',
                'verbose_name_plural': 'Agreement Type',
            },
        ),
        migrations.CreateModel(
            name='CurrencyTransform',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('factor', models.IntegerField(blank=True, null=True, verbose_name='Factor between From and To Currency')),
                ('from_currency', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltransformfromcurrency', to='crm.Currency', verbose_name='From Currency')),
            ],
            options={
                'verbose_name': 'Currency Transform',
                'verbose_name_plural': 'Currency Transforms',
            },
        ),
        migrations.CreateModel(
            name='Estimation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_from', models.DateField(verbose_name='Estimation From')),
                ('date_until', models.DateField(verbose_name='Estimation To')),
                ('amount', models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Amount')),
                ('reporting_period', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.ReportingPeriod', verbose_name='Reporting Period based on which the estimation was done')),
            ],
            options={
                'verbose_name': 'Estimation of Resource Consumption',
                'verbose_name_plural': 'Estimation of Resource Consumptions',
            },
        ),
        migrations.CreateModel(
            name='EstimationStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=250, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('is_obsolete', models.BooleanField(verbose_name='Status represents estimation is obsolete')),
            ],
            options={
                'verbose_name': 'Estimation Status',
                'verbose_name_plural': 'Estimation Status',
            },
        ),
        migrations.CreateModel(
            name='Resource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='ResourceManager',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.UserExtension', verbose_name='User')),
            ],
        ),
        migrations.CreateModel(
            name='ResourcePrice',
            fields=[
                ('price_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='crm.Price')),
            ],
            bases=('crm.price',),
        ),
        migrations.CreateModel(
            name='ResourceType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=300, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
            ],
            options={
                'verbose_name': 'Resource Link Type',
                'verbose_name_plural': 'Resource Link Type',
            },
        ),
        migrations.RemoveField(
            model_name='employeeassignmenttotask',
            name='employee',
        ),
        migrations.RemoveField(
            model_name='employeeassignmenttotask',
            name='task',
        ),
        migrations.AlterModelOptions(
            name='customergrouptransform',
            options={'verbose_name': 'Customer Group Price Transform', 'verbose_name_plural': 'Customer Group Price Transforms'},
        ),
        migrations.AlterModelOptions(
            name='product',
            options={},
        ),
        migrations.AlterModelOptions(
            name='unittransform',
            options={'verbose_name': 'Unit Transform', 'verbose_name_plural': 'Unit Transforms'},
        ),
        migrations.RenameField(
            model_name='genericprojectlink',
            old_name='task',
            new_name='project',
        ),
        migrations.RemoveField(
            model_name='task',
            name='planned_end_date',
        ),
        migrations.RemoveField(
            model_name='task',
            name='planned_start_date',
        ),
        migrations.AddField(
            model_name='project',
            name='default_currency',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='crm.Currency', verbose_name='Default Currency'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='customergrouptransform',
            name='from_customer_group',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltransfromfromcustomergroup', to='crm.CustomerGroup', verbose_name='From Customer Group'),
        ),
        migrations.AlterField(
            model_name='customergrouptransform',
            name='to_customer_group',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltransfromtocustomergroup', to='crm.CustomerGroup', verbose_name='To Customer Group'),
        ),
        migrations.AlterField(
            model_name='salesdocument',
            name='custom_date_field',
            field=models.DateField(blank=True, null=True, verbose_name='Custom Date'),
        ),
        migrations.CreateModel(
            name='HumanResource',
            fields=[
                ('resource_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='crm.Resource')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.UserExtension', verbose_name='User')),
            ],
            bases=('crm.resource',),
        ),
        migrations.DeleteModel(
            name='EmployeeAssignmentToTask',
        ),
        migrations.AddField(
            model_name='resourceprice',
            name='resource',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Resource', verbose_name='Resource'),
        ),
        migrations.AddField(
            model_name='resource',
            name='resource_manager',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.ResourceManager', verbose_name='Manager'),
        ),
        migrations.AddField(
            model_name='resource',
            name='resource_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.ResourceType', verbose_name='Resource Type'),
        ),
        migrations.AddField(
            model_name='estimation',
            name='resource',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Resource'),
        ),
        migrations.AddField(
            model_name='estimation',
            name='status',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.EstimationStatus', verbose_name='Status of the estimation'),
        ),
        migrations.AddField(
            model_name='estimation',
            name='task',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Task', verbose_name='Task'),
        ),
        migrations.AddField(
            model_name='currencytransform',
            name='to_currency',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltransformtocurrency', to='crm.Currency', verbose_name='To Currency'),
        ),
        migrations.AddField(
            model_name='agreement',
            name='costs',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.ResourcePrice'),
        ),
        migrations.AddField(
            model_name='agreement',
            name='resource',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Resource'),
        ),
        migrations.AddField(
            model_name='agreement',
            name='status',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.AgreementStatus'),
        ),
        migrations.AddField(
            model_name='agreement',
            name='task',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Task', verbose_name='Task'),
        ),
        migrations.AddField(
            model_name='agreement',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.AgreementType'),
        ),
        migrations.AddField(
            model_name='agreement',
            name='unit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Unit'),
        ),
        migrations.AddField(
            model_name='work',
            name='human_resource',
            field=models.ForeignKey(null=True, to='crm.HumanResource', on_delete=django.db.models.deletion.SET_NULL),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='currencytransform',
            name='product_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.ProductType', verbose_name='Product'),
        ),
        migrations.RunPython(initiate_human_resource, reverse_func),
        migrations.RemoveField(
            model_name='work',
            name='employee',
        ),
        migrations.AlterField(
            model_name='product',
            name='product_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.ProductType', verbose_name='Product Type'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='work',
            name='human_resource',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.HumanResource'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.crm.contact.phone_address import PhoneAddress
from koalixcrm.crm.contact.email_address import EmailAddress
from koalixcrm.crm.contact.postal_address import PostalAddress
from koalixcrm.crm.contact.call import Call
from koalixcrm.crm.contact.person import *
from koalixcrm.crm.const.purpose import *
from koalixcrm.global_support_functions import xstr
from koalixcrm.crm.inlinemixin import LimitedAdminInlineMixin


class Contact(models.Model):
    name = models.CharField(max_length=300,
                            verbose_name=_("Name"))
    date_of_creation = models.DateTimeField(verbose_name=_("Created at"),
                                            auto_now_add=True)
    last_modification = models.DateTimeField(verbose_name=_("Last modified"),
                                             auto_now=True)
    last_modified_by = models.ForeignKey('auth.User',
                                         on_delete=models.CASCADE,
                                         limit_choices_to={'is_staff': True},
                                         blank=True,
                                         verbose_name=_("Last modified by"),
                                         editable=True)

    class Meta:
        app_label = "crm"
        verbose_name = _('Contact')
        verbose_name_plural = _('Contact')

    def __str__(self):
        return self.name


class PhoneAddressForContact(PhoneAddress):
    purpose = models.CharField(verbose_name=_("Purpose"),
                               max_length=1,
                               choices=PURPOSESADDRESSINCUSTOMER)
    person = models.ForeignKey(Contact, on_delete=models.CASCADE)

    class Meta:
        app_label = "crm"
        verbose_name = _('Phone Address For Contact')
        verbose_name_plural = _('Phone Address For Contact')

    def __str__(self):
        return str(self.phone)


class EmailAddressForContact(EmailAddress):
    purpose = models.CharField(verbose_name=_("Purpose"),
                               max_length=1,
                               choices=PURPOSESADDRESSINCUSTOMER)
    person = models.ForeignKey(Contact, on_delete=models.CASCADE)

    class Meta:
        app_label = "crm"
        verbose_name = _('Email Address For Contact')
        verbose_name_plural = _('Email Address For Contact')

    def __str__(self):
        return str(self.email)


class PostalAddressForContact(PostalAddress):
    purpose = models.CharField(verbose_name=_("Purpose"),
                               max_length=1,
                               choices=PURPOSESADDRESSINCUSTOMER)
    person = models.ForeignKey(Contact, on_delete=models.CASCADE)

    class Meta:
        app_label = "crm"
        verbose_name = _('Postal Address For Contact')
        verbose_name_plural = _('Postal Address For Contact')

    def __str__(self):
        return xstr(self.pre_name) + ' ' + xstr(self.name) + ' ' + xstr(self.address_line_1)


class ContactPostalAddress(admin.StackedInline):
    model = PostalAddressForContact
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('prefix',
                       'pre_name',
                       'name',
                       'address_line_1',
                       'address_line_2',
                       'address_line_3',
                       'address_line_4',
                       'zip_code',
                       'town',
                       'state',
                       'country',
                       'purpose')
        }),
    )
    allow_add = True


class ContactPhoneAddress(admin.TabularInline):
    model = PhoneAddressForContact
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('phone', 'purpose',)
        }),
    )
    allow_add = True


class ContactEmailAddress(admin.TabularInline):
    model = EmailAddressForContact
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('email', 'purpose',)
        }),
    )
    allow_add = True


class ContactPersonAssociation(models.Model):
    contact = models.ForeignKey(Contact, on_delete=models.CASCADE, related_name='person_association', blank=True, null=True)
    person = models.ForeignKey(Person, on_delete=models.CASCADE, related_name='contact_association', blank=True, null=True)

    class Meta:
        app_label = "crm"
        verbose_name = _('Contacts')
        verbose_name_plural = _('Contacts')

    def __str__(self):
        return ''


class PeopleInlineAdmin(admin.TabularInline):
    model = ContactPersonAssociation
    extra = 0
    show_change_link = True


class CompaniesInlineAdmin(admin.TabularInline):
    model = ContactPersonAssociation
    extra = 0
    show_change_link = True


class OptionPerson(admin.ModelAdmin):
    list_display = ('id',
                    'name',
                    'pre_name',
                    'email',
                    'role',
                    'get_companies',)
    fieldsets = (('', {'fields': ('prefix',
                                  'name',
                                  'pre_name',
                                  'role',
                                  'email',
                                  'phone',)}),)
    allow_add = True
    inlines = [CompaniesInlineAdmin]

    def get_companies(self, obj):
        items = []
        for c in obj.companies.all():
            items.append(c.name)
        return ','.join(items)

    get_companies.short_description = _("Works at")


class CallForContact(Call):
    company = models.ForeignKey(Contact, on_delete=models.CASCADE)
    cperson = models.ForeignKey(Person, on_delete=models.CASCADE, verbose_name=_("Person"),
                                blank=True,
                                null=True)
    purpose = models.CharField(verbose_name=_("Purpose"),
                               max_length=1,
                               choices=PURPOSECALLINCUSTOMER)

    class Meta:
        app_label = "crm"
        verbose_name = _('Call')
        verbose_name_plural = _('Calls')

    def __str__(self):
        return xstr(self.description) + ' ' + xstr(self.date_due)


class VisitForContact(Call):
    company = models.ForeignKey(Contact, on_delete=models.CASCADE)
    cperson = models.ForeignKey(Person,
                                on_delete=models.CASCADE,
                                verbose_name=_("Person"),
                                blank=True,
                                null=True)
    purpose = models.CharField(verbose_name=_("Purpose"),
                               max_length=1,
                               choices=PURPOSEVISITINCUSTOMER)
    ref_call = models.ForeignKey(CallForContact,
                                 on_delete=models.CASCADE,
                                 verbose_name=_("Reference Call"),
                                 blank=True,
                                 null=True)

    class Meta:
        app_label = "crm"
        verbose_name = _('Visit')
        verbose_name_plural = _('Visits')

    def __str__(self):
        return xstr(self.description) + ' ' + xstr(self.date_due)


class ContactCall(LimitedAdminInlineMixin, admin.StackedInline):
    model = CallForContact
    extra = 0
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': (
                'description',
                'date_due',
                'purpose',
                'status',
                'cperson',)
        }),
    )
    allow_add = True

    def get_filters(self, request, obj):
        return getattr(self, 'filters', ()) if obj is None else (('cperson', dict(companies=obj.id)),)


class ContactVisit(LimitedAdminInlineMixin, admin.StackedInline):
    model = VisitForContact
    extra = 0
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': (
                'description',
                'date_due',
                'purpose',
                'status',
                'cperson',
                'ref_call',)
        }),
    )
    allow_add = True

    def get_filters(self, request, obj):
        return getattr(self, 'filters', ()) if obj is None else (('cperson', dict(companies=obj.id)),('ref_call', dict(company=obj.id, status='S')))


class StateFilter(admin.SimpleListFilter):
    title = _('State')
    parameter_name = 'state'

    def lookups(self, request, model_admin):
        items = []
        for a in PostalAddressForContact.objects.values('state').distinct():
            if a['state']:
                items.append((a['state'], _(a['state'])))
        return (
            items
        )

    def queryset(self, request, queryset):
        if self.value():
            matching_addresses = PostalAddressForContact.objects.filter(state=self.value())
            ids = [a.person.id for a in matching_addresses]
            return queryset.filter(pk__in=ids)
        return queryset


class CityFilter(admin.SimpleListFilter):
    title = _('City')
    parameter_name = 'city'

    def lookups(self, request, model_admin):
        items = []
        state = request.GET.get('state', None)
        unique_list = PostalAddressForContact.objects.all().order_by('town')
        adjusted_queryset = unique_list if state is None else unique_list.filter(state=state)
        for a in adjusted_queryset.values('town').distinct():
            if a['town']:
                items.append((a['town'], _(a['town'])))
        return (
            items
        )

    def queryset(self, request, queryset):
        if self.value():
            matching_addresses = PostalAddressForContact.objects.filter(town=self.value())
            ids = [(a.person.id) for a in matching_addresses]
            return queryset.filter(pk__in=ids)
        return queryset




from rest_framework import serializers

from koalixcrm.crm.product.unit import Unit


class OptionUnitJSONSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(required=False)
    description = serializers.CharField(read_only=True)
    shortName = serializers.CharField(source='short_name', read_only=True)

    class Meta:
        model = Unit
        fields = ('id',
                  'description',
                  'shortName')


class UnitJSONSerializer(serializers.HyperlinkedModelSerializer):
    shortName = serializers.CharField(source='short_name')
    description = serializers.CharField()
    isFractionOf = OptionUnitJSONSerializer(source='is_a_fraction_of',
                                            allow_null=True)
    fractionFactor = serializers.DecimalField(source='fraction_factor_to_next_higher_unit',
                                              max_digits=20,
                                              decimal_places=10,
                                              required=False,
                                              allow_null=True)

    class Meta:
        model = Unit
        fields = ('id',
                  'description',
                  'shortName',
                  'isFractionOf',
                  'fractionFactor')
        depth = 1

    def create(self, validated_data):
        unit = Unit()
        unit.description = validated_data['description']
        unit.short_name = validated_data['short_name']
        if 'fraction_factor_to_next_higher_unit' in validated_data:
            unit.fraction_factor_to_next_higher_unit = validated_data['fraction_factor_to_next_higher_unit']

        parent_unit = validated_data.pop('is_a_fraction_of')
        if parent_unit:
            if parent_unit.get('id', None):
                unit.is_a_fraction_of = Unit.objects.get(id=parent_unit.get('id', None))
            else:
                unit.is_a_fraction_of = None
        unit.save()
        return unit

    def update(self, instance, validated_data):
        instance.description = validated_data.get('description', instance.description)
        instance.short_name = validated_data.get('short_name', instance.short_name)

        parent_unit = validated_data.pop('is_a_fraction_of')
        if parent_unit:
            if parent_unit.get('id', instance.is_a_fraction_of_id):
                instance.is_a_fraction_of = Unit.objects.get(id=parent_unit.get('id', None))
            else:
                instance.is_a_fraction_of = instance.is_a_fraction_of_id
        else:
            instance.is_a_fraction_of = None
        instance.fraction_factor_to_next_higher_unit = validated_data.get('fraction_factor_to_next_higher_unit',
                                                                          instance.fraction_factor_to_next_higher_unit)
        instance.save()

        return instance



# -*- coding: utf-8 -*-
from rest_framework import serializers
from koalixcrm.crm.reporting.agreement import Agreement
from koalixcrm.crm.reporting.agreement_type import AgreementType
from koalixcrm.crm.reporting.agreement_status import AgreementStatus
from koalixcrm.crm.reporting.resource_price import ResourcePrice
from koalixcrm.crm.reporting.resource import Resource
from koalixcrm.crm.reporting.task import Task
from koalixcrm.crm.product.unit import Unit
from koalixcrm.crm.rest.task_rest import OptionTaskJSONSerializer
from koalixcrm.crm.rest.unit_rest import OptionUnitJSONSerializer
from koalixcrm.crm.rest.resource_price_rest import OptionResourcePriceJSONSerializer
from koalixcrm.crm.rest.agreement_type_rest import OptionAgreementTypeJSONSerializer
from koalixcrm.crm.rest.agreement_status_rest import OptionAgreementStatusJSONSerializer
from koalixcrm.crm.rest.resource_rest import OptionResourceJSONSerializer


class AgreementJSONSerializer(serializers.HyperlinkedModelSerializer):
    task = OptionTaskJSONSerializer(source='task', allow_null=False)
    resource = OptionResourceJSONSerializer(source='resource', allow_null=False)
    unit = OptionUnitJSONSerializer(source='unit', allow_null=False)
    type = OptionAgreementTypeJSONSerializer(source='type', allow_null=False)
    status = OptionAgreementStatusJSONSerializer(source='status', allow_null=False)
    costs = OptionResourcePriceJSONSerializer(source='costs', allow_null=False)

    class Meta:
        model = Agreement
        fields = ('amount',
                  'date_from',
                  'date_until',
                  'task',
                  'resource',
                  'unit',
                  'costs',
                  'type',
                  'status')

    def create(self, validated_data):
        agreement = Agreement()
        agreement.amount = validated_data['amount']
        agreement.date_from = validated_data['dateFrom']
        agreement.date_until = validated_data['dateUntil']
        # Deserialize task
        task = validated_data.pop('task')
        if task:
            if task.get('id', None):
                agreement.task = Task.objects.get(id=task.get('id', None))
            else:
                agreement.task = None
        # Deserialize resource
        resource = validated_data.pop('resource')
        if resource:
            if resource.get('id', None):
                agreement.resource = Resource.objects.get(id=resource.get('id', None))
            else:
                agreement.resource = None
        # Deserialize unit
        unit = validated_data.pop('unit')
        if unit:
            if unit.get('id', None):
                agreement.unit = Unit.objects.get(id=unit.get('id', None))
            else:
                agreement.unit = None
        # Deserialize costs
        costs = validated_data.pop('costs')
        if costs:
            if costs.get('id', None):
                agreement.costs = ResourcePrice.objects.get(id=costs.get('id', None))
            else:
                agreement.costs = None
        # Deserialize type
        type = validated_data.pop('type')
        if type:
            if type.get('id', None):
                agreement.type = AgreementType.objects.get(id=type.get('id', None))
            else:
                agreement.type = None
        # Deserialize status
        status = validated_data.pop('status')
        if status:
            if type.get('id', None):
                agreement.status = AgreementStatus.objects.get(id=status.get('id', None))
            else:
                agreement.status = None

        agreement.save()
        return agreement

    def update(self, agreement, validated_data):
        agreement.amount = validated_data['amount']
        agreement.date_from = validated_data['dateFrom']
        agreement.date_until = validated_data['dateUntil']
        task = validated_data.pop('task')
        if task:
            if task.get('id', agreement.task):
                agreement.task = Task.objects.get(id=task.get('id', None))
            else:
                agreement.task = agreement.task_id
        else:
            agreement.task = None
        # Deserialize resource
        resource = validated_data.pop('resource')
        if resource:
            if resource.get('id', agreement.resource):
                agreement.resource = Resource.objects.get(id=resource.get('id', None))
            else:
                agreement.resource = agreement.resource_id
        else:
            agreement.resource = None
        # Deserialize unit
        unit = validated_data.pop('unit')
        if task:
            if unit.get('id', agreement.unit):
                agreement.unit = Unit.objects.get(id=unit.get('id', None))
            else:
                agreement.unit = agreement.unit_id
        else:
            agreement.unit = None
        # Deserialize costs
        costs = validated_data.pop('costs')
        if costs:
            if costs.get('id', agreement.costs):
                agreement.costs = ResourcePrice.objects.get(id=costs.get('id', None))
            else:
                agreement.costs = agreement.costs_id
        else:
            agreement.costs = None
        # Deserialize type
        type = validated_data.pop('type')
        if type:
            if type.get('id', agreement.type):
                agreement.type = Task.objects.get(id=type.get('id', None))
            else:
                agreement.type = agreement.type_id
        else:
            agreement.type = None
        # Deserialize status
        status = validated_data.pop('status')
        if status:
            if status.get('id', agreement.status):
                agreement.status = AgreementStatus.objects.get(id=status.get('id', None))
            else:
                agreement.status = agreement.status_id
        else:
            agreement.status = None

        agreement.save()

        return agreement






# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-11 18:35
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0027_auto_20180606_2034'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='planned_end_date',
            field=models.DateField(blank=True, null=True, verbose_name='Planned End Date'),
        ),
        migrations.AlterField(
            model_name='task',
            name='planned_start_date',
            field=models.DateField(blank=True, null=True, verbose_name='Planned Start Date'),
        ),
        migrations.AlterField(
            model_name='task',
            name='status',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.TaskStatus', verbose_name='Task Status'),
        ),
    ]



from rest_framework import serializers
from koalixcrm.djangoUserExtension.user_extension.user_extension import UserExtension
from koalixcrm.crm.rest.currency_rest import CurrencyJSONSerializer
from koalixcrm.djangoUserExtension.rest.template_set_rest import TemplateSetJSONSerializer


class OptionUserExtensionJSONSerializer(serializers.HyperlinkedModelSerializer):
    defaultTemplateSet = TemplateSetJSONSerializer(source='default_template_set')
    defaultCurrency = CurrencyJSONSerializer(source='default_currency')

    class Meta:
        model = UserExtension
        fields = ('id',
                  'user',
                  'defaultTemplateSet',
                  'defaultCurrency')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-04 21:11
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0008_auto_20180104_2042'),
    ]

    operations = [
        migrations.RenameField(
            model_name='contract',
            old_name='dateofcreation',
            new_name='date_of_creation',
        ),
        migrations.RenameField(
            model_name='contract',
            old_name='defaultcurrency',
            new_name='default_currency',
        ),
        migrations.RenameField(
            model_name='contract',
            old_name='defaultcustomer',
            new_name='default_customer',
        ),
        migrations.RenameField(
            model_name='contract',
            old_name='defaultSupplier',
            new_name='default_supplier',
        ),
        migrations.RenameField(
            model_name='contract',
            old_name='lastmodification',
            new_name='last_modification',
        ),
        migrations.RenameField(
            model_name='contract',
            old_name='lastmodifiedby',
            new_name='last_modified_by',
        ),
        migrations.RenameField(
            model_name='invoice',
            old_name='derivatedFromQuote',
            new_name='derived_from_quote',
        ),
        migrations.RenameField(
            model_name='invoice',
            old_name='payableuntil',
            new_name='payable_until',
        ),
        migrations.RenameField(
            model_name='invoice',
            old_name='paymentBankReference',
            new_name='payment_bank_reference',
        ),
        migrations.RenameField(
            model_name='position',
            old_name='lastCalculatedPrice',
            new_name='last_calculated_price',
        ),
        migrations.RenameField(
            model_name='position',
            old_name='lastCalculatedTax',
            new_name='last_calculated_tax',
        ),
        migrations.RenameField(
            model_name='position',
            old_name='lastPricingDate',
            new_name='last_pricing_date',
        ),
        migrations.RenameField(
            model_name='position',
            old_name='overwriteProductPrice',
            new_name='overwrite_product_price',
        ),
        migrations.RenameField(
            model_name='position',
            old_name='positionNumber',
            new_name='position_number',
        ),
        migrations.RenameField(
            model_name='position',
            old_name='sentOn',
            new_name='sent_on',
        ),
        migrations.RenameField(
            model_name='product',
            old_name='accoutingProductCategorie',
            new_name='accounting_product_categorie',
        ),
        migrations.RenameField(
            model_name='product',
            old_name='dateofcreation',
            new_name='date_of_creation',
        ),
        migrations.RenameField(
            model_name='product',
            old_name='defaultunit',
            new_name='default_unit',
        ),
        migrations.RenameField(
            model_name='product',
            old_name='lastmodification',
            new_name='last_modification',
        ),
        migrations.RenameField(
            model_name='product',
            old_name='lastmodifiedby',
            new_name='last_modified_by',
        ),
        migrations.RenameField(
            model_name='product',
            old_name='productNumber',
            new_name='product_number',
        ),
        migrations.RenameField(
            model_name='purchaseconfirmation',
            old_name='validuntil',
            new_name='valid_until',
        ),
        migrations.RenameField(
            model_name='purchaseorder',
            old_name='dateofcreation',
            new_name='date_of_creation',
        ),
        migrations.RenameField(
            model_name='purchaseorder',
            old_name='externalReference',
            new_name='external_reference',
        ),
        migrations.RenameField(
            model_name='purchaseorder',
            old_name='lastCalculatedPrice',
            new_name='last_calculated_price',
        ),
        migrations.RenameField(
            model_name='purchaseorder',
            old_name='lastCalculatedTax',
            new_name='last_calculated_tax',
        ),
        migrations.RenameField(
            model_name='purchaseorder',
            old_name='lastmodification',
            new_name='last_modification',
        ),
        migrations.RenameField(
            model_name='purchaseorder',
            old_name='lastmodifiedby',
            new_name='last_modified_by',
        ),
        migrations.RenameField(
            model_name='purchaseorder',
            old_name='lastPricingDate',
            new_name='last_pricing_date',
        ),
        migrations.RenameField(
            model_name='quote',
            old_name='validuntil',
            new_name='valid_until',
        ),
        migrations.RenameField(
            model_name='salescontract',
            old_name='dateofcreation',
            new_name='date_of_creation',
        ),
        migrations.RenameField(
            model_name='salescontract',
            old_name='externalReference',
            new_name='external_reference',
        ),
        migrations.RenameField(
            model_name='salescontract',
            old_name='lastCalculatedPrice',
            new_name='last_calculated_price',
        ),
        migrations.RenameField(
            model_name='salescontract',
            old_name='lastCalculatedTax',
            new_name='last_calculated_tax',
        ),
        migrations.RenameField(
            model_name='salescontract',
            old_name='lastmodification',
            new_name='last_modification',
        ),
        migrations.RenameField(
            model_name='salescontract',
            old_name='lastmodifiedby',
            new_name='last_modified_by',
        ),
        migrations.RenameField(
            model_name='salescontract',
            old_name='lastPricingDate',
            new_name='last_pricing_date',
        ),
    ]



# -*- coding: utf-8 -*-

import factory
from koalixcrm.accounting.models import ProductCategory


class StandardProductCategoryFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProductCategory
        django_get_or_create = ('title',)

    title = "Service"
    profit_account = ""
    loss_account = ""



from django.test import TestCase
from koalixcrm.global_support_functions import get_string_between
from subprocess import Popen, PIPE
from koalixcrm.version import KOALIXCRM_VERSION
import pytest


class VersionIncreaseTest(TestCase):
    @staticmethod
    def get_all_koalixcrm_version_from_pip():
        process_out = Popen(['pip install koalix-crm=='], shell=True, stderr=PIPE)
        output, string_containing_all_version = process_out.communicate()
        all_koalixcrm_version_csv = get_string_between(string_containing_all_version.__str__(), "koalix-crm== (", ")")
        all_koalixcrm_version = all_koalixcrm_version_csv.split(", ")
        return all_koalixcrm_version

    def setUp(self):
        self.available_versions = VersionIncreaseTest.get_all_koalixcrm_version_from_pip()

    @pytest.mark.version_increase
    def test_version_increase(self):
        last_version = ""
        for version in self.available_versions:
            last_version = version
            if version == KOALIXCRM_VERSION:
                break
        self.assertNotEqual(last_version, KOALIXCRM_VERSION)




# -*- coding: utf-8 -*-

import factory
import datetime
from koalixcrm.crm.models import ResourcePrice
from koalixcrm.crm.factories.factory_resource import StandardResourceFactory
from koalixcrm.crm.factories.factory_unit import StandardUnitFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.global_support_functions import make_date_utc


class StandardResourcePriceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ResourcePrice

    resource = factory.SubFactory(StandardResourceFactory)
    unit = factory.SubFactory(StandardUnitFactory)
    currency = factory.SubFactory(StandardCurrencyFactory)
    customer_group = factory.SubFactory(StandardCustomerGroupFactory)
    price = "100.50"
    valid_from = make_date_utc(datetime.datetime(2018, 6, 15, 00))
    valid_until = make_date_utc(datetime.datetime(2024, 6, 15, 00))


class HighResourcePriceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ResourcePrice

    resource = factory.SubFactory(StandardResourceFactory)
    unit = factory.SubFactory(StandardUnitFactory)
    currency = factory.SubFactory(StandardCurrencyFactory)
    customer_group = factory.SubFactory(StandardCustomerGroupFactory)
    price = "250.50"
    valid_from = make_date_utc(datetime.datetime(2018, 6, 15, 00))
    valid_until = make_date_utc(datetime.datetime(2024, 6, 15, 00))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-04-22 21:05
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0004_auto_20171210_2126'),
        ('accounting', '0006_auto_20180422_2048'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='accountingperiod',
            name='template_set',
        ),
        migrations.AddField(
            model_name='accountingperiod',
            name='template_profit_loss_statement',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_profit_loss_statement_template_set', to='djangoUserExtension.DocumentTemplate', verbose_name='Referred template for profit, loss statement'),
        ),
        migrations.AddField(
            model_name='accountingperiod',
            name='template_set_balance_sheet',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_balancesheet_template_set', to='djangoUserExtension.DocumentTemplate', verbose_name='Referred template for balance sheet'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-08-05 20:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0045_auto_20180805_2047'),
    ]
    operations = [
        migrations.RemoveField(
            model_name='work',
            name='worked_hours')
    ]



from rest_framework import serializers
from koalixcrm.crm.reporting.estimation_status import EstimationStatus


class OptionEstimationStatusJSONSerializer(serializers.HyperlinkedModelSerializer):
    title = serializers.CharField(source='title', read_only=True)
    description = serializers.CharField(source='description', read_only=True)
    isObsolete = serializers.BooleanField(source='is_obsolete', read_only=True)

    class Meta:
        model = EstimationStatus
        fields = ('title',
                  'description',
                  'is_obsolete')


class EstimationStatusJSONSerializer(serializers.HyperlinkedModelSerializer):
    title = serializers.CharField(source='title')
    description = serializers.CharField(source='description')
    isObsolete = serializers.BooleanField(source='is_obsolete')

    class Meta:
        model = EstimationStatus
        fields = ('title',
                  'description',
                  'is_obsolete')



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.product.customer_group_transform import CustomerGroupTransform
from koalixcrm.crm.factories.factory_product_type import StandardProductTypeFactory
from koalixcrm.crm.factories.factory_customer_group import AdvancedCustomerGroupFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory


class StandardCustomerGroupTransformFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = CustomerGroupTransform
    from_customer_group = factory.SubFactory(AdvancedCustomerGroupFactory)
    to_customer_group = factory.SubFactory(StandardCustomerGroupFactory)
    product_type = factory.SubFactory(StandardProductTypeFactory)
    factor = "10"



"""
Django base settings for test_koalixcrm project.

Generated by 'django-admin startproject' using Django 1.11.3.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""
import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(__file__))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'modify_during_deployment'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

# Application definition
PREREQUISITE_APPS = [
    'django.contrib.contenttypes',
    'grappelli.dashboard',
    'grappelli',
    'filebrowser',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'rest_framework',
    'django_filters'
]

PROJECT_APPS = [
    'koalixcrm.crm',
    'koalixcrm.accounting',
    'koalixcrm.djangoUserExtension',
    'koalixcrm.subscriptions',
]

INSTALLED_APPS = PREREQUISITE_APPS + PROJECT_APPS

KOALIXCRM_PLUGINS = (
    'koalixcrm.subscriptions',
)

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'koalixcrm.crm.middleware.timezoneMiddleware.TimezoneMiddleware',
]

ROOT_URLCONF = 'projectsettings.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'projectsettings.wsgi.application'


# Password validation
# https://docs.djangoproject.com/en/1.11/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static')

MEDIA_URL = "/media/"
MEDIA_ROOT = os.path.join(BASE_DIR)

PROJECT_ROOT = BASE_DIR

# Settings specific for koalixcrm
PDF_OUTPUT_ROOT = os.path.join(STATIC_ROOT, 'pdf')

# Settings specific for filebrowser
FILEBROWSER_DIRECTORY = 'media/uploads/'
FILEBROWSER_EXTENSIONS = {
    'XML': ['.xml'],
    'XSL': ['.xsl'],
    'JPG': ['.jpg'],
    'PNG': ['.png'],
    'GIF': ['.gif'],
    'TTF': ['.ttf'],
}

LOGIN_URL = "/admin/login"

REST_FRAMEWORK = {
    'DEFAULT_FILTER_BACKENDS': ('django_filters.rest_framework.DjangoFilterBackend',),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.SessionAuthentication',
    )
}



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _


class CurrencyTransform(models.Model):
    from_currency = models.ForeignKey('Currency',
                                      on_delete=models.CASCADE,
                                      blank=False,
                                      null=False,
                                      verbose_name=_("From Currency"),
                                      related_name="db_reltransformfromcurrency")
    to_currency = models.ForeignKey('Currency',
                                    on_delete=models.CASCADE,
                                    blank=False,
                                    null=False,
                                    verbose_name=_("To Currency"),
                                    related_name="db_reltransformtocurrency")
    product_type = models.ForeignKey('ProductType',
                                     on_delete=models.CASCADE,
                                     blank=False,
                                     null=False,
                                     verbose_name=_("Product"))
    factor = models.DecimalField(verbose_name=_("Factor between From and To Currency"),
                                 blank=False,
                                 null=False,
                                 max_digits=17,
                                 decimal_places=2,)

    def get_transform_factor(self):
        return self.factor

    def __str__(self):
        return "From " + self.from_currency.short_name + " to " + self.to_currency.short_name

    class Meta:
        app_label = "crm"
        verbose_name = _('Currency Transform')
        verbose_name_plural = _('Currency Transforms')


class CurrencyTransformInlineAdminView(admin.TabularInline):
    model = CurrencyTransform
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('', {
            'fields': ('from_currency',
                       'to_currency',
                       'factor',)
        }),
    )
    allow_add = True
